@@ -24,7 +24,11 @@
 
 #include <IOKit/IOKitLib.h>
 #include <libkern/OSAtomic.h>
+#include <sys/stat.h>
+
+#include <cerrno>
 #include <cmath>
+#include <fstream>
 #include <iomanip>
 #include <iostream>
 #include <limits>
@@ -382,10 +386,36 @@ kern_return_t SmcAccessor::PrintAll() {
   return kIOReturnSuccess;
 }
 
+SmcTemp::SmcTemp(bool isFailSoft)
+    : is_fail_soft_(isFailSoft) {
+  if (is_fail_soft_) {
+    if (mkdir(storage_path_.c_str(), 0777) && errno != EEXIST) {
+      std::cerr << "Failed to create directory: " << storage_path_ << std::endl;
+    }
+  }
+}
+
 bool SmcTemp::IsValidTemperature(double temperature, const std::pair<unsigned int, unsigned int>& limits) {
   return temperature > limits.first && temperature < limits.second;
 }
 
+bool SmcTemp::StoreValidTemperature(double temperature, std::string file_name) {
+  if (!is_fail_soft_) {
+    return false;
+  }
+  std::ofstream out_file(storage_path_ + file_name);
+
+  if (!out_file) {
+    std::cerr << "Failed to open the file: " << storage_path_ + file_name << std::endl;
+    return false;
+  }
+
+  out_file << temperature;
+  out_file.close();
+
+  return true;
+}
+
 double SmcTemp::CalculateAverageTemperature(const std::vector<std::string>& sensors,
                                      const std::pair<unsigned int, unsigned int>& limits) {
   double temp = 0.0;
@@ -411,18 +441,22 @@ double SmcTemp::GetCpuTemp() {
   // https://github.com/narugit/smctemp/issues/2
   temp = smc_accessor_.ReadValue(kSensorTC0D);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
+    StoreValidTemperature(temp, cpu_file_);
     return temp;
   }
   temp = smc_accessor_.ReadValue(kSensorTC0E);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
+    StoreValidTemperature(temp, cpu_file_);
     return temp;
   }
   temp = smc_accessor_.ReadValue(kSensorTC0F);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
+    StoreValidTemperature(temp, cpu_file_);
     return temp;
   }
   temp = smc_accessor_.ReadValue(kSensorTC0P);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
+    StoreValidTemperature(temp, cpu_file_);
     return temp;
   }
 #elif defined(ARCH_TYPE_ARM64)
@@ -431,7 +465,13 @@ double SmcTemp::GetCpuTemp() {
   const std::pair<unsigned int, unsigned int> valid_temperature_limits{10, 120};
 
   const std::string cpumodel = getCPUModel();
-  if (cpumodel.find("m3") != std::string::npos) {  // Apple M2
+  if (cpumodel.find("m4") != std::string::npos) {  // Apple M4
+    sensors.emplace_back(static_cast<std::string>(kSensorTp01));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp09));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0f));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp05));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0D));
+  } else if (cpumodel.find("m3") != std::string::npos) {  // Apple M3
     // CPU core 1
     sensors.emplace_back(static_cast<std::string>(kSensorTp01));
     // CPU core 2
@@ -449,6 +489,12 @@ double SmcTemp::GetCpuTemp() {
     // CPU core 8
     sensors.emplace_back(static_cast<std::string>(kSensorTp0r));
   } else if (cpumodel.find("m2") != std::string::npos) {  // Apple M2
+    // CPU efficient cores 1 through 4 on M2 Max 12 Core Chip
+    sensors.emplace_back(static_cast<std::string>(kSensorTp1h));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp1t));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp1p));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp1l));
+
     // CPU core 1
     sensors.emplace_back(static_cast<std::string>(kSensorTp01));
     // CPU core 2
@@ -498,11 +544,18 @@ double SmcTemp::GetCpuTemp() {
 
   temp = CalculateAverageTemperature(sensors, valid_temperature_limits);
   if (temp > std::numeric_limits<double>::epsilon()) {
+    if (IsValidTemperature(temp, valid_temperature_limits)) {
+      StoreValidTemperature(temp, cpu_file_);
+    }
     return temp;
   }
 
   temp = CalculateAverageTemperature(aux_sensors, valid_temperature_limits);
+  if (IsValidTemperature(temp, valid_temperature_limits)) {
+    StoreValidTemperature(temp, cpu_file_);
+  }
 #endif
+
   return temp;
 }
 
@@ -512,17 +565,24 @@ double SmcTemp::GetGpuTemp() {
   const std::pair<unsigned int, unsigned int> valid_temperature_limits{0, 110};
   temp = smc_accessor_.ReadValue(kSensorTG0D);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
+    StoreValidTemperature(temp, gpu_file_);
     return temp;
   }
   temp = smc_accessor_.ReadValue(kSensorTPCD);
-    if (IsValidTemperature(temp, valid_temperature_limits)) {
+  if (IsValidTemperature(temp, valid_temperature_limits)) {
+    StoreValidTemperature(temp, gpu_file_);
     return temp;
   }
 #elif defined(ARCH_TYPE_ARM64)
   std::vector<std::string> sensors;
   const std::pair<unsigned int, unsigned int> valid_temperature_limits{10, 120};
   const std::string cpumodel = getCPUModel();
-  if (cpumodel.find("m3") != std::string::npos) {  // Apple M3
+  if (cpumodel.find("m4") != std::string::npos) {  // Apple M4
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0D));  // GPU 1
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0P));  // GPU 2
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0X));  // GPU 3
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0j));  // GPU 4
+  } else if (cpumodel.find("m3") != std::string::npos) {  // Apple M3
     sensors.emplace_back(static_cast<std::string>(kSensorTg0D));  // GPU 1
     sensors.emplace_back(static_cast<std::string>(kSensorTg0P));  // GPU 2
     sensors.emplace_back(static_cast<std::string>(kSensorTg0X));  // GPU 3
@@ -547,9 +607,45 @@ double SmcTemp::GetGpuTemp() {
     return temp;
   }
   temp = CalculateAverageTemperature(sensors, valid_temperature_limits);
+  if (IsValidTemperature(temp, valid_temperature_limits)) {
+    StoreValidTemperature(temp, gpu_file_);
+  }
 #endif
   return temp;
 }
 
+double SmcTemp::GetLastValidCpuTemp() {
+  std::string file_path = storage_path_ + cpu_file_;
+  std::ifstream file(file_path);
+  if (!file.is_open()) {
+    std::cerr << "Failed to open the file: " << file_path << std::endl;
+  }
+
+  double value;
+  file >> value;
+
+  if (file.fail()) {
+    std::cerr << "Failed to read sensor value from file: " + file_path << std::endl;
+  }
+
+  return value;
+}
+
+double SmcTemp::GetLastValidGpuTemp() {
+  std::string file_path = storage_path_ + gpu_file_;
+  std::ifstream file(file_path);
+  if (!file.is_open()) {
+    std::cerr << "Failed to open the file: " << file_path << std::endl;
+  }
+
+  double value;
+  file >> value;
+
+  if (file.fail()) {
+    std::cerr << "Failed to read sensor value from file: " + file_path << std::endl;
+  }
+
+  return value;
+}
 }
 
