@@ -33,7 +33,7 @@
 #define COUNT_OF(x) ((sizeof(x)/sizeof(0[x])) / ((size_t)(!(sizeof(x) % sizeof(0[x])))))
 
 namespace smctemp {
-const std::string kVersion = "0.3.2";
+const char kVersion[] = "0.6.0";
 constexpr char kIOAppleSmcHiddenClassName[] = "AppleSMC";
 constexpr char kSmcCmdReadBytes = 5;
 constexpr char kSmcCmdReadIndex = 8;
@@ -76,6 +76,10 @@ constexpr UInt32Char_t kSensorTp0f = "Tp0f";
 constexpr UInt32Char_t kSensorTp0j = "Tp0j";
 constexpr UInt32Char_t kSensorTp0n = "Tp0n";
 constexpr UInt32Char_t kSensorTp0r = "Tp0r";
+constexpr UInt32Char_t kSensorTp1h = "Tp1h";
+constexpr UInt32Char_t kSensorTp1t = "Tp1t";
+constexpr UInt32Char_t kSensorTp1p = "Tp1p";
+constexpr UInt32Char_t kSensorTp1l = "Tp1l";
 // GPU
 constexpr UInt32Char_t kSensorTg05 = "Tg05";
 constexpr UInt32Char_t kSensorTg0D = "Tg0D";
@@ -115,14 +119,21 @@ class SmcTemp {
  private:
   double CalculateAverageTemperature(const std::vector<std::string>& sensors,
                                      const std::pair<unsigned int, unsigned int>& limits);
-  bool IsValidTemperature(double temperature, const std::pair<unsigned int, unsigned int>& limits);
+  bool StoreValidTemperature(double temperature, std::string file_name);
   SmcAccessor smc_accessor_;
+  bool is_fail_soft_;
+  const std::string storage_path_ = "/tmp/smctemp/";
+  const std::string cpu_file_ = "cpu_temperature.txt";
+  const std::string gpu_file_ = "gpu_temperature.txt";
 
  public:
-  SmcTemp() = default;
+  explicit SmcTemp(bool isFailSoft);
   ~SmcTemp() = default;
   double GetCpuTemp();
   double GetGpuTemp();
+  double GetLastValidCpuTemp();
+  double GetLastValidGpuTemp();
+  bool IsValidTemperature(double temperature, const std::pair<unsigned int, unsigned int>& limits);
 };
 
 typedef struct {
